#
# Conditional build:
%bcond_without	static	# static version of mksh
%bcond_without	tests	# rtchecks and test.sh checks

Summary:	MirBSD Korn Shell
Summary(pl.UTF-8):	Powłoka Korna z MirBSD
Name:		mksh
Version:	50d
Release:	1
License:	BSD
Group:		Applications/Shells
Source0:	http://www.mirbsd.org/MirOS/dist/mir/mksh/%{name}-R%{version}.tgz
# Source0-md5:	1c3882c07a760b23df1ad94ad0b4ed2e
Source1:	%{name}-mkshrc
Patch0:		%{name}-mkshrc_support.patch
Patch1:		%{name}-circumflex.patch
Patch2:		%{name}-no_stop_alias.patch
Patch3:		%{name}-cmdline-length.patch
URL:		https://www.mirbsd.org/mksh.htm
%if %{with tests}
BuildRequires:	ed
BuildRequires:	perl-base
%endif
%{?with_static:BuildRequires:   glibc-static}
BuildRequires:	rpmbuild(macros) >= 1.670
# is needed for /etc directory existence
Requires(pre):	FHS
Requires:	setup >= 2.4.6-2
Obsoletes:	pdksh
BuildRoot:	%{tmpdir}/%{name}-%{version}-root-%(id -u -n)

%define		_bindir			/bin

%description
mksh is the MirBSD enhanced version of the Public Domain Korn shell
(pdksh), a Bourne-compatible shell which is largely similar to the
original AT&T Korn shell. It includes bug fixes and feature
improvements in order to produce a modern, robust shell good for
interactive and especially script use. It has UTF-8 support in the
emacs command line editing mode; corresponds to OpenBSD 4.2-current
ksh sans GNU bash-like $PS1; the build environment requirements are
autoconfigured; throughout code simplification/bugfix/enhancement has
been done, and the shell has extended compatibility to other modern
shells.

%description -l pl.UTF-8
mksh to pochodząca z MirBSD rozszerzona wersja powłoki Public Domain
Korn Shell (pdksh) - kompatybilnej z powłoką Bourne'a, w większości
zbliżonej do oryginalnej powłoki Korna z AT&T. Zawiera poprawki błędów
i rozszerzenia mające na celu stworzenie współczesnej powłoki o
bogatych możliwościach do użytku interaktywnego i (zwłaszcza) w
skryptach. Ma obsługę UTF-8 w trybie edycji linii poleceń w stylu
emacsa; $PS1 odpowiada temu z ksh obecnym w OpenBSD 4.2-current;
środowisko budowania jest automatycznie konfigurowane; dzięki
wykonanym uproszczeniom kodu, poprawkom i rozszerzeniom powłoka ma
rozszerzoną kompatybilność z innymi współczesnymi powłokami.

%package static
Summary:	Statically linked the MirBSD enhanced version of pdksh
Summary(pl.UTF-8):	Skonsolidowana statycznie powłoka mksh
Group:		Applications/Shells
# requires base for /etc/mkshrc?
Requires:	%{name} = %{version}-%{release}

%description static
mksh is the MirBSD enhanced version of the Public Domain Korn shell
(pdksh), a Bourne-compatible shell which is largely similar to the
original AT&T Korn shell.

This packege contains statically linked version of mksh.

%description static -l pl.UTF-8
mksh to pochodząca z MirBSD rozszerzona wersja powłoki Public Domain
Korn Shell (pdksh) - kompatybilnej z powłoką Bourne'a, w większości
zbliżonej do oryginalnej powłoki Korna z AT&T.

W tym pakiecie jest mksh skonsolidowany statycznie.

%prep
%setup -qcT
gzip -dc %{SOURCE0} | cpio -mid
mv mksh/* .; rmdir mksh

%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1

# sed rules instead of patch (needed update for every release)
sed -i -e 's|\(#define.*MKSH_VERSION.*\)"|\1 @DISTRO@"|g' sh.h
sed -i -e 's|\(@(#)MIRBSD KSH.*\)|\1 @DISTRO@|g' check.t

# fill distro
sed -i -e 's#@DISTRO@#PLD/Linux 3.0#g' check.t sh.h
# sanity checks
grep PLD/Linux check.t || exit 1
grep PLD/Linux sh.h || exit 1

# we'll need this later due to -DMKSH_GCC55009
cat >rtchecks <<'EOF'
typeset -i sari=0
typeset -Ui uari=0
typeset -i x=0
print -r -- $((x++)):$sari=$uari.
let --sari --uari
print -r -- $((x++)):$sari=$uari.
sari=2147483647 uari=2147483647
print -r -- $((x++)):$sari=$uari.
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
let --sari --uari
let 'sari *= 2' 'uari *= 2'
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
sari=-2147483648 uari=-2147483648
print -r -- $((x++)):$sari=$uari.
let --sari --uari
print -r -- $((x++)):$sari=$uari.
EOF

cat >rtchecks.expected <<'EOF'
0:0=0.
1:-1=4294967295.
2:2147483647=2147483647.
3:-2147483648=2147483648.
4:-1=4294967295.
5:0=0.
6:-2147483648=2147483648.
7:2147483647=2147483647.
EOF

%build
install -d out

CC="%{__cc}" \
CFLAGS="%{rpmcflags} -DMKSH_GCC55009" \
LDFLAGS="%{rpmldflags}" \
CPPFLAGS="%{rpmcppflags}" \
sh ./Build.sh -Q -r -j -c lto

# skip some tests if not on terminal
if ! tty -s; then
	skip_tests="-C regress:no-ctty"
fi

%if %{with tests}
./mksh rtchecks >rtchecks.got 2>&1
if ! cmp --quiet rtchecks.got rtchecks.expected ; then
	echo "rtchecks failed"
	diff -Naurp rtchecks.expected rtchecks.got
	exit 1
fi
./test.sh -v $skip_tests
%endif
mv mksh out/mksh.dynamic

%if %{with static}
CC="%{__cc}" \
CFLAGS="%{rpmcflags} -DMKSH_GCC55009" \
LDFLAGS="%{rpmldflags} -static" \
CPPFLAGS="%{rpmcppflags}" \
sh ./Build.sh -Q -r -j -c lto

%if %{with tests}
./test.sh -v $skip_tests
./mksh rtchecks >rtchecks.got 2>&1
if ! cmp --quiet rtchecks.got rtchecks.expected ; then
	echo "rtchecks failed"
	diff -Naurp %{SOURCE3} rtchecks.got
	exit 1
fi
%endif
mv mksh out/mksh.static
%endif

%install
rm -rf $RPM_BUILD_ROOT
install -d $RPM_BUILD_ROOT{%{_bindir},%{_mandir}/man1}
install -p out/mksh.dynamic $RPM_BUILD_ROOT%{_bindir}/mksh
%{?with_static:install -p out/mksh.static $RPM_BUILD_ROOT%{_bindir}/mksh.static}

cp -a mksh.1 $RPM_BUILD_ROOT%{_mandir}/man1/mksh.1
echo ".so mksh.1" > $RPM_BUILD_ROOT%{_mandir}/man1/sh.1

install -D %{SOURCE1} $RPM_BUILD_ROOT%{_sysconfdir}/mkshrc
ln -sf mksh $RPM_BUILD_ROOT%{_bindir}/sh

# some pdksh scripts used that
ln -sf mksh $RPM_BUILD_ROOT%{_bindir}/ksh

%clean
rm -rf $RPM_BUILD_ROOT

%post   -p %add_etc_shells -p /bin/sh /bin/ksh /bin/mksh
%preun  -p %remove_etc_shells -p /bin/sh /bin/ksh /bin/mksh

%post	static -p %add_etc_shells -p /bin/mksh.static
%preun	static -p %remove_etc_shells -p /bin/mksh.static

%triggerpostun -p <lua> -- pdksh
if arg[2] ~= 0 then
%lua_add_etc_shells /bin/sh /bin/ksh /bin/mksh
end

%files
%defattr(644,root,root,755)
%doc dot.mkshrc
%config(noreplace,missingok) %verify(not md5 mtime size) %{_sysconfdir}/mkshrc
%attr(755,root,root) %{_bindir}/mksh
%attr(755,root,root) %{_bindir}/ksh
%attr(755,root,root) %{_bindir}/sh
%{_mandir}/man1/mksh.1*
%{_mandir}/man1/sh.1*

%if %{with static}
%files static
%defattr(644,root,root,755)
%attr(755,root,root) %{_bindir}/mksh.static
%endif
%changelog
* Tue Oct 07 2014 PLD Linux Team <feedback@pld-linux.org>
- For complete changelog see: http://git.pld-linux.org/?p=packages/mksh.git;a=log;h=master

* Tue Oct 07 2014 Arkadiusz Miśkiewicz <arekm@maven.pl> 625811d
- up to 50d

* Sat Oct 04 2014 Arkadiusz Miśkiewicz <arekm@maven.pl> 28c118b
- up to 50c; this is low impact SECURITY bug fix release (unrelated to shellshock bugs)

* Sat Sep 06 2014 Arkadiusz Miśkiewicz <arekm@maven.pl> e9da756
- up to 50b

* Thu Aug 21 2014 Jakub Bogusz <qboosh@pld-linux.org> e42128b
- updated to 50
- updated mkshrc_support patch

* Mon Jan 20 2014 Arkadiusz Miśkiewicz <arekm@maven.pl> a07aa89
- rel 1; fix test suite with our circumflex patch

* Mon Jan 20 2014 Elan Ruusamäe <glen@delfi.ee> caf7897
- rel 0.1 by baggins request

* Mon Jan 20 2014 Elan Ruusamäe <glen@delfi.ee> d236736
- up to 49; 1 test fails

* Tue Oct 15 2013 Arkadiusz Miśkiewicz <arekm@maven.pl> 00d9479
- up to 48b

* Wed Aug 07 2013 Jakub Bogusz <qboosh@pld-linux.org> 7660cf0
- use simpler, direct, rpm4/rpm5 compliant macro

* Tue Aug 06 2013 Elan Ruusamäe <glen@delfi.ee> 1c7e5b2
- do pdksh migration via trigger

* Mon Aug 05 2013 Elan Ruusamäe <glen@delfi.ee> 20fe49c
- manually merge changes from mksh-44 branch

* Thu Aug 01 2013 Marcin Krol <hawk@tld-linux.org> f4608d8
- updated to R47

* Wed Jun 26 2013 Jan Rękorajski <baggins@pld-linux.org> 7486e2e
- rel back to 0.1, https://bugs.launchpad.net/mksh/+bug/1179287

* Wed Jun 26 2013 Jan Rękorajski <baggins@pld-linux.org> 6bf9e2c
- add also mksh in posttrans
- rel 2

* Tue May 07 2013 Jakub Bogusz <qboosh@pld-linux.org> 570f989
- updated to 46

* Tue Apr 30 2013 Kacper Kornet <draenog@pld-linux.org> 83f423e
- up to 45

* Thu Mar 07 2013 Arkadiusz Miśkiewicz <arekm@maven.pl> 1024574
- up to 44

* Wed Feb 20 2013 Kacper Kornet <draenog@pld-linux.org> 23dc9b6
- up to 43

* Sat Feb 16 2013 Jakub Bogusz <qboosh@pld-linux.org> de8a6e5
- bcond descs

* Fri Feb 15 2013 Arkadiusz Miśkiewicz <arekm@maven.pl> 90672aa
- up to 42b

